"use strict";
const gulp = require("gulp");
const paths = require("./paths");
const sourcemaps = require("gulp-sourcemaps");
const babel = require("gulp-babel");
const bableOptions = require("./babeloptions");
const through2 = require("through2");
const flow = require('gulp-flowtype');
const notify = require('gulp-notify');

let targetFormat = "systemjs";

function buildSource(options, path) {
    return new Promise((resolve) => {
        gulp.src(paths.sourceFiles)
            .pipe(sourcemaps.init())
            .pipe(flow({
                all: false,
                weak: false,
                declarations: './declarations',
                killFlow: false,
                beep: true,
                abort: false
            }))
            .pipe(babel(options))
            .on('error', notify.onError('<%= error.message %>'))
            .pipe(sourcemaps.write('.'))
            .pipe(gulp.dest(path ? path : paths.sourceDestPath))
            .on("end", () => {resolve()});
    });
}

function buildTests(options) {
    return new Promise((resolve) => {
        gulp.src(paths.testFiles)
            .pipe(sourcemaps.init())
            .pipe(flow({
                all: false,
                weak: false,
                declarations: './declarations',
                killFlow: false,
                beep: true,
                abort: false
            }))
            .pipe(babel(options))
            .on('error', notify.onError('<%= error.message %>'))
            .pipe(sourcemaps.write('.'))
            .pipe(gulp.dest(paths.testDestPath))
            .on("end", () => {resolve()});
    });
}

function getModule(file, encoding, callback) {
    let object = JSON.parse(String(file.contents));

    const jspm = object.jspm;
    if (!jspm) {
        callback();
    }

    const format = jspm.format;

    if (format) {
        targetFormat = format;
    }

    callback();
}


gulp.task('build:source', ['prebuild-setmodule'], function () {
    return buildSource(bableOptions(targetFormat));
});

gulp.task('build:tests', ['clean:app'], function () {
    const sourcePormise = buildSource(bableOptions("commonjs"));
    const testPromise = buildTests(bableOptions("commonjs"));

    return Promise.all([sourcePormise, testPromise]);
});

gulp.task('prebuild-setmodule', function() {
    return gulp.src("package.json")
        .pipe(through2({objectMode: true}, getModule))
});

gulp.task('build:dist', function() {
    buildSource(bableOptions("amd"), paths.distAmd);
    buildSource(bableOptions("commonjs"), paths.distCommonjs);
    buildSource(bableOptions("systemjs"), paths.distSystemjs);
});
