"use strict";

const fetch = require("node-fetch");
const fs = require("fs");

class Github {
    constructor() {
        this.config = null;

        if (fs.existsSync('githubconfig.json'))
        {
            const text = fs.readFileSync('githubconfig.json', 'utf8');
            this.config = JSON.parse(text);
        }
        else {
            console.error('please make sure you have a githubconfig.json in the project root');
        }
    }

    createCommitPath(sha) {
        return `${this.config.host}/repos/${this.config.repoUserName}/${this.config.repoName}/commits/${sha}`;
    }

    getFilesOfCommit(sha) {
        const path = this.createCommitPath(sha);

        let result = {
            html: [],
            source: [],
            tests: []
        };

        return new Promise((resolve => {
            fetch(path)
                .then(function(res) {
                    return res.json();
                })
                .then(function(json) {
                    for(let i = 0; i < json.files.length; i++) {
                        const fileName = json.files[i].filename;

                        if (this.isHtmlFile(fileName)) {
                            result.html.push(fileName)
                        }
                        else if (this.isSource(fileName)) {
                            result.source.push(fileName);
                        }
                        else if (this.isTest(fileName)) {
                            result.tests.push(fileName);
                        }
                    }

                    resolve(result);
                }.bind(this));
        }).bind(this));

    }

    isHtmlFile(fileName) {
        return fileName.indexOf('.html') > -1;
    }

    isSource(fileName) {
        return fileName.indexOf('src/') > -1 && fileName.indexOf('.ts') > -1;
    }

    isTest(fileName) {
        return fileName.indexOf('test/') > -1 && fileName.indexOf('.ts') > -1;
    }
}

module.exports = Github;