"use strict";

const fs = require("fs");

class TestResults {
    constructor() {
        this.passed = [];
        this.failed = [];
        this.pending = [];
    }

    logFail(test, error) {
        this.failed.push({test: test, error: error});
        console.error('fail: %s', test.fullTitle());
        console.error(error.stack);
    }

    logPassed(test) {
        this.passed.push(test);
        console.log('pass: %s', test.fullTitle());
    }

    logPending(test) {
        this.pending.push(test);
    }

    getReportObject() {
        let result = {};

        result.numberOfTests = this.passed.length + this.pending.length + this.failed.length;
        result.numberOfFailedTests = this.failed.length;

        if (this.pending.length > 0) {
            result.pending = [];

            for (let test of this.pending) {
                result.pending.push({
                    file: test.file,
                    title: test.title
                });
            }
        }

        if (this.failed.length > 0) {
            result.failed = [];

            for (let test of this.failed) {
                result.failed.push({
                    file: test.test.file,
                    title: test.test.title,
                    error: test.error.message,
                    stack: test.error.stack
                })
            }
        }

        return result;
    }

    saveReportToFile() {
        var dir = './coverage';

        if (!fs.existsSync(dir)){
            fs.mkdirSync(dir);
        }

        var fileName = dir +  '/test-results.json';
        var textToSave = JSON.stringify(this.getReportObject(), null, 4);

        fs.writeFileSync(fileName, textToSave);
    }
}


function TestReporter(runner) {
    const testResults = new TestResults();

    runner.on('pass', function(test){
        testResults.logPassed(test);
    });

    runner.on('fail', function(test, error) {
        testResults.logFail(test, error);
    });

    runner.on('pending', function(test) {
        testResults.logPending(test);
    });

    runner.on('end', function(){
        testResults.saveReportToFile();
    });
}

module.exports = TestReporter;
