"use strict";
var gulp = require("gulp");
var rename = require("gulp-rename");
var through2 = require("through2");
var bump = require('gulp-bump');
var publishScope = null;
var uglify = require('gulp-uglify');

class Publish {
    /*
     publish options:
     src     : required
     dest    : required
     rename  : optional
     minify  : optional
     */
    publish(options) {
        if (!options || !Array.isArray(options)) {
            console.error('require publish options in publish method');
            return;
        }

        options.forEach(function (option) {
            var stream = gulp.src(option.src);

            if (option.rename) {
                stream = stream.pipe(rename(option.rename));
            }

            if (option.minify) {
                stream = stream.pipe(uglify());
            }

            stream.pipe(gulp.dest(option.dest));
        });
    }

    /*
     setPackage options:
     src     : required
     dest    : required
     */
    setPackage(options) {
        if(!options) {
            console.error('require setPackage options')
        }

        publishScope = options.scope ? options.scope : null;

        gulp.src(options.src)
            .pipe(bump())
            .pipe(gulp.dest('./'))
            .pipe(through2({objectMode: true}, this.cleanPackage))
            .pipe(gulp.dest(options.dest));
    };

    /*
     Used to clean package.json file for publication
     */
    cleanPackage (file, encoding, callback) {
        var object = JSON.parse(String(file.contents));
        var result = file;

        delete object.devDependencies;
        if (publishScope) {
            object.name = "@" + publishScope + "/" + object.name;
        }

        var stringResult = JSON.stringify(object, null, '\t');

        result.contents = new Buffer(stringResult);
        this.push(result);
        callback();
    }

    publishJspm() {

    }
}





module.exports = Publish;

/*

build in operators for scss

 */