"use strict";

class AuditController {
    constructor() {
        const missing_tests_link = document.getElementById("missing_tests");
        const html_audit_link = document.getElementById("html_audit");
        const code_coverage_link = document.getElementById("code_coverage");
        const e2e_link = document.getElementById("e2e");
        const tests_link = document.getElementById("tests");

        const missing_tests_text = this.extractDOM(missing_tests_link);
        const html_audit_text = this.extractDOM(html_audit_link);
        const code_coverage_text = this.extractDOM(code_coverage_link);
        const e2e_container_text = this.extractDOM(e2e_link);
        const tests_container_text = this.extractDOM(tests_link);

        if (missing_tests_text.length > 0) {
            document.getElementById("missing_tests_container").innerHTML = missing_tests_text;
        }

        if (html_audit_text.length > 0) {
            document.getElementById("html_audit_container").innerHTML = html_audit_text;
        }

        if (code_coverage_text.length > 0) {
            document.getElementById("code_coverage_container").innerHTML = code_coverage_text;
        }

        if (e2e_container_text.length > 0) {
            document.getElementById("e2e_container").innerHTML = e2e_container_text;
        }

        if (tests_container_text.length > 0) {
            document.getElementById("tests_container").innerHTML = tests_container_text;
        }
    }

    extractDOM(link) {
        return link ? link.import.querySelector("template").innerHTML : "";
    }

    hideAllElements() {
        const missingTestsContainer = document.getElementById("missing_tests_container");
        const htmlAuditContainer = document.getElementById("html_audit_container");
        const codeCoverageContainer = document.getElementById("code_coverage_container");
        const e2eContainer = document.getElementById("e2e_container");
        const e2eDetailsContainer = document.getElementById("e2e_details_container");
        const testsContainer = document.getElementById("tests_container");

        auditController.makeInvisible(missingTestsContainer);
        auditController.makeInvisible(htmlAuditContainer);
        auditController.makeInvisible(codeCoverageContainer);
        auditController.makeInvisible(e2eContainer);
        auditController.makeInvisible(e2eDetailsContainer);
        auditController.makeInvisible(testsContainer);
    }

    menuitem_selected(event) {
        this.hideAllElements();

        const id = event.currentTarget.getAttribute("data-target");
        auditController.makeVisible(document.getElementById(id));
    }

    makeVisible(element) {
        element.removeAttribute("hidden");
        element.removeAttribute("auria-hidden");
    }

    makeInvisible(element) {
        element.setAttribute("hidden", "hidden");
        element.setAttribute("auria-hidden", "true");
    }

    showDetail(event) {
        const id = event.currentTarget.getAttribute("href").split(".html")[0];
        const link = document.getElementById(id);
        const dom = this.extractDOM(link);

        this.hideAllElements();
        const e2eDetailsContainer = document.getElementById("e2e_details_container");
        e2eDetailsContainer.innerHTML = dom;
        this.makeVisible(e2eDetailsContainer);
    }
}

const auditController = new AuditController();


